/*
**	Sybase Registered Procedure Client Application
**	Confidential Property of Sybase, Inc.
**	(c) Copyright Sybase, Inc. 1993
**	All rights reserved
**
**  CTUTILS
**	This file contains utility routines used by the client sample 
**	programs.
**
**	The routines contained in this file are:
**
**      server_err_handler()    An Open Server error handler routine.
**      client_err_handler()    An Open Client error handler routine.
**      cs_err_handler()        A CS-Library error handler routine.
**
*/

/*
** Operating system header files required by this application.
*/
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<ctype.h>

/*
** SYBASE header files required by Open Client. 
*/
#include	<ctpublic.h>
#include	<ctsample.h>

#if 	(USE_SCCSID)
static char     Sccsid[] = {"%Z% %M% %I% %G%"};
#endif	/* (USE_SCCSID) */

/*
**  server_err_handler - Server errors 
**	Open Client calls this routine when it receives messages from the
**	server. It reads the message information and prints it to the
**	console. 
**
**  Parameters:
**	cp		A pointer to the context structure for this 
**			application.
**
**	chp		A pointer to the connection handle on which the 
**			error occurred.	(It can be NULL.)
**
**	msgp		A pointer to a structure containing information 
**			about the server message.
**  Returns:
**	CS_SUCCEED
*/
CS_RETCODE CS_PUBLIC server_err_handler(cp, chp, msgp)
CS_CONTEXT      *cp;
CS_CONNECTION   *chp;
CS_SERVERMSG    *msgp;
{
	/* 
	** If chp is (CS_CONNECTION *)NULL, this error occurred when no 
	** connection handle was active.  Just return cancel.  
	*/
	if (chp == (CS_CONNECTION *)NULL) 
		return(CS_SUCCEED);

	/* Build message for the client.				*/
	switch((int)msgp->svrnlen)
	{
	case CS_NULLTERM:
		/* Server name already null-terminated.			*/
		break;
	case 0:
		/* Dummy remote server name.				*/
		(CS_VOID)strcpy(msgp->svrname, "unknown");
		break;
	default:
		/* Null terminate remote server name.			*/
		if (msgp->svrnlen < (CS_MAX_NAME - 1))
			msgp->svrname[msgp->svrnlen] = '\0';
		else
			msgp->svrname[CS_MAX_NAME - 1] = '\0';

		break;
	}

	/* Build and print the client message. 				*/
	(CS_VOID)fprintf(stderr, 
		"Server: msgno(%d) severity(%d) state(%d)\n",
		msgp->msgnumber, msgp->severity, msgp->state);

	/* Null terminate message.					*/
	switch((int)msgp->textlen)
	{
	case CS_NULLTERM:
		/* Message already null terminated.			*/
		break;
	case CS_UNUSED:
	case 0:
		/* No message specified.				*/
		msgp->text[0] = '\0';
		break;
	default:
		if (msgp->textlen < (CS_MAX_MSG - 1))
			msgp->text[msgp->textlen] = '\0';
		else
			msgp->text[CS_MAX_MSG - 1] = '\0';
		break;
	}

	/* Print the null terminated message. 				*/
	(CS_VOID)fprintf(stderr, "\t%s\n", msgp->text);

	return(CS_SUCCEED);
}

/*
**  client_err_handler - Open Client errors.
**	This is the Open Client error handler.  All Open Client connections
**	use this error handler.  The error messages are printed to the 
**	console.
**
**  Parameters:
**	chp		A pointer to the connection handle on which this
**			error occurred. (It can be NULL.)
**
**	cmdp		A pointer to the command handle active when error 
**			occurred. (It can be NULL.)
**
**	emsgp		A pointer to an Open Client CS_CLIENTMSG structure.
**
**  Returns:
**	CS_SUCCEED
*/
CS_RETCODE CS_PUBLIC client_err_handler(chp, cmdp, emsgp)
CS_CONNECTION	*chp;
CS_COMMAND	*cmdp;
CS_CLIENTMSG	*emsgp;
{
	/* 
	** If chp is (CS_CONNECTION *)NULL, this error occurred when no 
	** connection handle was active.  Just return cancel.  
	*/
	if (chp == (CS_CONNECTION *)NULL) 
		return (CS_SUCCEED);

	/*
	** Format and print the message. */
	(CS_VOID)fprintf(stderr, 
		"Server: error(%d) serverity(%d) OS error(%d)\n",
		emsgp->msgnumber, emsgp->severity, emsgp->osnumber);

	(CS_VOID)fprintf(stderr, "\t%s\n", emsgp->msgstring);

	/*
	** If there was an operating system error, send the error
	** message text for the OS error.
	 */
	if ((emsgp->osstringlen != 0) && (emsgp->osstringlen != CS_UNUSED))
	{
		(CS_VOID)fprintf(stderr, "Server OS Error: %d/%d/%d\n", 
			emsgp->msgnumber, emsgp->severity, emsgp->osnumber);
		(CS_VOID)fprintf(stderr, "%s\n", emsgp->osstring);
	}
	return (CS_SUCCEED);
}

/*
**  CS_ERR_HANDLER - CS-Library error handler.
** 	This routine is the CS-Library error handler used by this
** 	application. It is called by CS-Library whenever an error
** 	occurs. Here, we simply print the error and return.
**
**  Parameters:
**      cp              A pointer to the connection handle for connection
**		 	on which the message was received.
** 			(It can be NULL.)
**
**      msg             The structure containing information about the
**			server message.
**
**  Returns:
**      CS_SUCCEED
*/
CS_RETCODE CS_PUBLIC cs_err_handler(cp, msg)
CS_CONTEXT	*cp;
CS_CLIENTMSG	*msg;
{
	/*
	** Build the error message string to be logged.
	*/
	(CS_VOID)fprintf(stderr, 
		"CS-Library error %ld/%ld/%ld/%ld - %s\n",
		CS_LAYER(msg->msgnumber), 
		CS_ORIGIN(msg->msgnumber), 
		CS_SEVERITY(msg->msgnumber), 
		CS_NUMBER(msg->msgnumber), 
		msg->msgstring);

	/*
	** Print any operating system error information.
	*/
	if( msg->osstringlen > 0 )
	{
		(CS_VOID)fprintf(stderr, 
			"CS-Library OS error %ld - %s.\n",
			msg->osnumber, msg->osstring);
	}

	/*
	** All done.
	*/
	return (CS_SUCCEED);
}
