
/*
**
**	Sybase Open Server/International Sample
**	Confidential Property of Sybase, Inc.
**	(c) Copyright Sybase, Inc. 1993
**	All rights reserved
**
**  INTLCHAR
**	This program accepts language commands from a client, parses them,
**	and then executes them appropriately.  It also sends and
**	receives LANGUAGE and CHARSET option commands from a client.
**
**	This program demonstrates some of Open Server 10.0's 
**	internationalization features. Clients can execute "set langauge" 
**	and/or "set charset" commands via isql or get/set/clear the language 
**	and charset.  The application maps these requests to actions 
**	against the thread's locale definition.
**
**  Usage:
**	intlchar [server name] [trace flags]
**
**              If the server name argument is given, it is used.  Otherwise
**              the value of the DSLISTEN environment variable is used.
**              The optional trace flags result in diagnostic information 
**              being written to the Open Server log file. 
**
**  Language Commands:
**
** 	set language @language		Set the language of the thread.  
**
** 	set charset @charset		Set the character set of the thread.  
**
**	sp_serverinfo @request, @arg1	Execute the sp_serverinfo registered 
**					stored procedure.
**		Valid @request values: 
**
**		server_csname		Return the thread's character set .
**		server_soname   	Return the thread's sort order. 
**	  	csdefinition @arg1	Return the character set definition
**					for @arg1 (a valid character set).
**	  	sodefinition @arg1	Return the sort order definition for
**					@arg1 (a valid sort).
**
**  Returns:
**      0 on successful execution.
**      1 on error.
*/

/*
** Operating system header files required by Open Server.
*/
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<ctype.h>

/*
** SYBASE header files required by Open Server and Open Client.
*/
#include	<ospublic.h>
#include	<oserror.h>
#include	<ossample.h>

#if 	(USE_SCCSID)
static char     Sccsid[] = {"%Z% %M% %I% %G%"};
#endif	/* (USE_SCCSID) */

/*
** Open Server user defined message codes.
*/
#define	CMD_FAIL	(CS_INT)SRV_MAXERROR+1	/* command failed.	*/
#define	LOCALE_INFO	(CS_INT)SRV_MAXERROR+2	/* locale change.	*/
#define	OPTION_FAIL	(CS_INT)SRV_MAXERROR+3	/* option failed.	*/

/*
** Command parsing macros.
*/
#define WHITESPACE(b,l,m)	{ while (isspace(b[l]) && (l < m)) l++; }
#define UPTOSPACE(b,l,m)	{ while (!isspace(b[l]) && (b[l] != ',') && \
				   	 (l < m)) l++; }

/*
** Useful defines.
*/
#define CMD_LANGUAGE		"set language"
#define CMD_CHARSET		"set charset"
#define CMD_SERVERINFO		"sp_serverinfo"
#define CMD_PARAM1		"@request"
#define CMD_PARAM2		"@arg1"

#define	COPY_ALL		(CS_INT)(-1)

/*
** Prototypes.
*/
CS_RETCODE CS_PUBLIC lang_handler PROTOTYPE ((
	SRV_PROC	*srvproc
	));
CS_RETCODE CS_PUBLIC option_handler PROTOTYPE ((
	SRV_PROC	*srvproc
	));
CS_RETCODE get_locale_mapping PROTOTYPE((
	CS_CONTEXT	*cp,
	SRV_PROC	*srvproc,
	CS_INT		type,
	CS_CHAR		*name,
	CS_INT		len
	));
CS_RETCODE set_locale PROTOTYPE((
	CS_CONTEXT	*cp,
	SRV_PROC	*srvproc,
	CS_INT		type,
	CS_CHAR		*name,
	CS_INT		len
	));
CS_RETCODE lang_exec PROTOTYPE((
	CS_CONTEXT	*cp,
	SRV_PROC	*srvproc,
	CS_CHAR		*cmd,
	CS_INT		len
	));
CS_RETCODE exec_locale PROTOTYPE((
	CS_CONTEXT	*cp,
	SRV_PROC	*srvproc,
	CS_INT		type,
	CS_CHAR		*cmd,
	CS_INT		cur,
	CS_INT		len 
	));
CS_RETCODE exec_serverinfo PROTOTYPE((
	SRV_PROC	*srvproc,
	CS_CHAR		*cmd,
	CS_INT		cur,
	CS_INT		len
	));

int	main(argc, argv)
int	argc;
char	*argv[];
{
	CS_INT		rv;		 /* Value to return to 	env.	*/
	CS_BOOL		boolval;
	CS_CHAR		sname[CS_MAX_CHAR];
	CS_CHAR		logfile[CS_MAX_CHAR]; /* Log file name.		*/
	CS_CONTEXT	*cp;
	CS_CHAR		*servername;    /* Open Server application name.*/

	/* Initialization.						*/
	rv = 0;				/* Assume non-error exit.	*/

	/*
	** Initialize `servername' to the command line argument provided.
	*/
	servername = (CS_CHAR *)argv[1];

	/*
	** Allocate the context structure, and initialize the Open Server
	** application.	
	*/

	/* Allocate the context structure.				*/
	if (cs_ctx_global(CS_VERSION_100, &cp) != CS_SUCCEED)
	{
		(CS_VOID)fprintf(stderr, "cs_ctx_alloc() failed.  Exiting.\n");
		exit(1);
	}

	/* Install a CS-Library error handler.  			*/
	if (cs_config(cp, CS_SET, CS_MESSAGE_CB, (CS_VOID *)cs_err_handler, 
		CS_UNUSED, NULL) != CS_SUCCEED)
	{
		/*
		** Release the context structure already allocated.
		*/
		(CS_VOID)cs_ctx_drop(cp);

		(CS_VOID)fprintf(stderr, "cs_config() failed.  Exiting.\n");
		exit(1);
	}

	/*
	** Define the Open Server application behavior and the CS_CONTEXT
	** to be used.
	*/
	if (srv_version(cp, CS_VERSION_100) != CS_SUCCEED)
	{
		/*
		** Release the context structure already allocated.
		*/
		(CS_VOID)cs_ctx_drop(cp);

		(CS_VOID)fprintf(stderr, "srv_version() failed.  Exiting.\n");
		exit(1);
	}

	/* Parse the command line arguments.				*/
	if (proc_args(cp, argc, argv, sname, CS_MAX_CHAR,
		(CS_CHAR *)NULL, (CS_INT)0) != CS_SUCCEED)
	{
		/*
		** Release the context structure already allocated.
		*/
		(CS_VOID)cs_ctx_drop(cp);
		exit(1);
	}

	/*
	** Build a new Open Server log file name using `servername'.
	*/
	(CS_VOID)sprintf(logfile, "%s.log", servername);

	/*
	** Set the new log file name using the global SRV_S_LOGFILE
	** property.
	*/
	if(srv_props(cp, CS_SET, SRV_S_LOGFILE, logfile, CS_NULLTERM,
		(CS_INT *)NULL) == CS_FAIL)
	{
		/*
		** Release the context structure already allocated.
		*/
		(CS_VOID)cs_ctx_drop(cp);

		(CS_VOID)fprintf(stderr,
			"%s: srv_props(SRV_S_LOGFILE) failed\n", servername);
		exit(1);
	}

	/* Install the Open Server error handler.			*/
	if (srv_props(cp, CS_SET, SRV_S_ERRHANDLE, 
		(CS_VOID *)server_err_handler, CS_SIZEOF(CS_VOID *), 
		(CS_INT *)NULL) != CS_SUCCEED)
	{
		/*
		** Release the context structure already allocated.
		*/
		(CS_VOID)cs_ctx_drop(cp);

		(CS_VOID)fprintf(stderr, 
			"srv_props(ERRHANDLE) failed.  Exiting.\n");
		exit(1);
	}

	/* Initialize the Open Server to use the client's language	*/
	boolval = CS_FALSE;
	if (srv_props(cp, CS_SET, SRV_S_USESRVLANG, (CS_VOID *)&boolval, 
		CS_SIZEOF(boolval), (CS_INT *)NULL) != CS_SUCCEED)
	{
		/*
		** Release the context structure already allocated.
		*/
		(CS_VOID)cs_ctx_drop(cp);

		(CS_VOID)fprintf(stderr, 
			"srv_props(USESRVLANG) failed.  Exiting.\n");
		exit(1);
	}


	/* Initialize Open Server.					*/
	if (srv_init((SRV_CONFIG *)NULL, sname, CS_NULLTERM) 
		== (SRV_SERVER *)NULL)
	{
		/*
		** Release the context structure already allocated.
		*/
		(CS_VOID)cs_ctx_drop(cp);

		(CS_VOID)fprintf(stderr, "srv_init failed.  Exiting.\n");
		exit(1);
	}

	/*
	** Start Open Server . Get the  version string of the Open Server
	** application and null terminate it. Write the version string to
	** stderr and to the log file.
	*/
	print_version(cp, sname);

	/* Install the event handlers.					*/
	if ( srv_handle((SRV_SERVER *)NULL, SRV_START, stop_regproc)
		== (SRV_EVENTHANDLE_FUNC)NULL )
	{
		(CS_VOID)cs_ctx_drop(cp);
		(CS_VOID)fprintf(stderr, "srv_handle() failed. Exiting.\n");
		exit(1);
	}

	if (srv_handle((SRV_SERVER *)NULL, SRV_LANGUAGE, lang_handler) 
		== (SRV_EVENTHANDLE_FUNC)NULL)
	{
		/*
		** Release the context structure already allocated.
		*/
		(CS_VOID)cs_ctx_drop(cp);

		(CS_VOID)fprintf(stderr, "srv_handle() failed. Exiting.\n");
		exit(1);
	}
	if (srv_handle((SRV_SERVER *)NULL, SRV_OPTION, option_handler) 
		== (SRV_EVENTHANDLE_FUNC)NULL)
	{
		/*
		** Release the context structure already allocated.
		*/
		(CS_VOID)cs_ctx_drop(cp);

		(CS_VOID)fprintf(stderr, "srv_handle() failed. Exiting.\n");
		exit(1);
	}

	/*
	** srv_run returns CS_SUCCEED when stopped normally via a SRV_STOP
	** event.
	*/
	if (srv_run((SRV_SERVER *)NULL) != CS_SUCCEED) 
	{
		(CS_VOID)fprintf(stderr, "srv_run() failed. Exiting.\n");
		rv = 1;
	}

	/* Release the allocated control structures.			*/
	(CS_VOID)cs_ctx_drop(cp);
	exit(rv);
}

/*
**  OPTION_HANDLER - Option handler
**	Handler for SRV_OPTION events. 
**
**  Parameters:
**	srvproc - A pointer to an internal thread control structure.
**
**  Returns:
**	CS_SUCCEED | CS_FAIL
*/
CS_RETCODE	CS_PUBLIC option_handler(srvproc)
SRV_PROC	*srvproc;
{
	CS_INT		status;
	CS_INT		optcmd;		/* option command.		*/
	CS_INT		option;		/* clients requested option.	*/
	CS_INT		outlen;
	CS_BOOL		ok;
	CS_CHAR		optval[1024];
	CS_CONTEXT	*cp;

	/* Initialization. 						*/
	ok = CS_TRUE;
	status = SRV_DONE_FINAL;

	/* Get the option, type of option command, option value, etc.	 */
	if (srv_options(srvproc, CS_GET, &optcmd, &option, optval,
		CS_SIZEOF(optval), &outlen) != CS_SUCCEED)
	{
		ok = CS_FALSE;
	}

	/* Allocate a context structure.				*/
	if (ok && cs_ctx_global(CS_VERSION_100, &cp) != CS_SUCCEED)
		ok = CS_FALSE;

	if (!ok)
	{
		/*
		** Set the option command to a value that will pass through 
		** the switch and generate an error message and DONE_ERROR
		** packet.
		*/
		optcmd = 0;
	}

	switch((int)optcmd) 
	{
	case SRV_SETOPTION:
		switch((int)option)
		{
		case CS_OPT_NATLANG:
			/* 
			** Reset the locale to the locale matching the
			** language passed.
			*/
			if (set_locale(cp, srvproc, CS_SYB_LANG, optval, 
				outlen) != CS_SUCCEED)
			{
				ok = CS_FALSE;
			}
			break;
		case CS_OPT_CHARSET:
			/* 
			** Reset the locale to the locale matching the
			** character set passed.
			*/
			if (set_locale(cp, srvproc, CS_SYB_CHARSET, optval, 
				outlen) != CS_SUCCEED)
			{
				ok = CS_FALSE;
			}
			break;
		default:
			ok = CS_FALSE;
			break;
		}
		break;
	case SRV_GETOPTION:
		switch((int)option)
		{
		case CS_OPT_NATLANG:
			/* Determine the locale/language mapping.	*/
			if (get_locale_mapping(cp, srvproc, CS_SYB_LANG, 
				optval, 1024) == CS_SUCCEED)
			{
				/* Send the language name to the client.*/
				optcmd = SRV_SENDOPTION;
				if (srv_options(srvproc, CS_SET, 
					&optcmd, &option, optval, CS_NULLTERM, 
					(CS_INT *)NULL) != CS_SUCCEED)
				{
					ok = CS_FALSE;
				}
			}
			else
			{
				(CS_VOID)fprintf(stderr, 
					"option_handle() Locale map failed.\n");
				ok = CS_FALSE;
			}
			break;
		case CS_OPT_CHARSET:
			/* Determine the locale/charset mapping.*/
			if (get_locale_mapping(cp, srvproc, CS_SYB_CHARSET, 
				optval, 1024) == CS_SUCCEED)
			{
				/* Send the language name to the client.*/
				optcmd = SRV_SENDOPTION;
				if (srv_options(srvproc, CS_SET, &optcmd, 
					&option, optval, CS_NULLTERM, 
					(CS_INT *)NULL) != CS_SUCCEED)
				{
					ok = CS_FALSE;
				}
			}
			else
			{
				ok = CS_FALSE;
				(CS_VOID)fprintf(stderr, 
					"option_handle() Locale map failed.\n");
			}
			break;
		default:
			ok = CS_FALSE;

		}
		break;
	case SRV_CLEAROPTION:
		switch((int)option)
		{
		case CS_OPT_NATLANG:
		case CS_OPT_CHARSET:
			/*
			** Clear the language/charset by resetting the
			** thread's locale to the locale of the server.
			*/
			if (srv_thread_props(srvproc, CS_CLEAR, SRV_T_LOCALE, 
				(CS_LOCALE *)NULL, CS_UNUSED, (CS_INT *)NULL) 
				!= CS_SUCCEED)
			{
				ok = CS_FALSE;
			}
			break;
		default:
			ok = CS_FALSE;
		}
		break;
	default:
		ok = CS_FALSE;
		break;
	}

	if (!ok)
	{
		(CS_VOID)fprintf(stderr, 
			"option_handle() Unable to set/get option\n");
		status |= SRV_DONE_ERROR;
	}

	/* Send the final done packet to the client.			*/
	if (srv_senddone(srvproc, status, CS_TRAN_COMPLETED, 0) != CS_SUCCEED)
		(CS_VOID)fprintf(stderr, "srv_senddone() failed.\n");

	return (ok ? CS_SUCCEED : CS_FAIL);
}

/*
**  LANG_HANDLER - Language handler.
**	Execute one of a limited number of client language requests.
**	Notify the client of any unrecognized requests.
**
**  Parameters:
**	srvproc - A pointer to an internal thread control structure.
**
**  Returns:
**	CS_SUCCEED | CS_FAIL
*/
CS_RETCODE	CS_PUBLIC lang_handler(srvproc)
SRV_PROC	*srvproc;
{
	CS_BOOL		ok;		/* Error control variable.	*/
	CS_INT		size;		/* Size of language request.	*/
	CS_CHAR		*cmd;		/* Language buffer.		*/
	CS_CONTEXT	*cp;

	/* Initialization						*/
	ok = CS_TRUE;			/* All is okay.			*/
	cmd = (CS_CHAR *)NULL;		/* No language buffer yet.	*/

	/* Determine the size of the language string.			*/
	if ((size = srv_langlen(srvproc)) < 0) 
		ok = CS_FALSE;

	/* Allocate language string buffer.  Leave room for NULL. */
	if (ok && ((cmd = (CS_CHAR *)srv_alloc(size+1)) == (CS_CHAR *)NULL))
		ok = CS_FALSE;

	/* Copy language string into allocated buffer.			*/
	if (ok && (srv_langcpy(srvproc, (CS_INT)0, COPY_ALL, cmd) < 0)) 
		ok = CS_FALSE;

	/* Allocate a context structure.				*/
	if (ok && cs_ctx_global(CS_VERSION_100, &cp) != CS_SUCCEED)
		ok = CS_FALSE;

	/*
	** Execute the language request.
	*/
	if (ok && lang_exec(cp, srvproc, cmd, size) != CS_SUCCEED)
		ok = CS_FALSE;

	if (!ok)
	{
		/* Send the DONE_ERROR to the client.			*/
		if (srv_senddone(srvproc, SRV_DONE_FINAL | SRV_DONE_ERROR, 
			CS_TRAN_COMPLETED, 0) != CS_SUCCEED)
		{
			(CS_VOID)fprintf(stderr, "srv_senddone() failed.\n");
		}
	}

	/* Release all allocated handles and memory.			*/
	if (cmd != (CS_CHAR *)NULL) 
		(CS_VOID)srv_free(cmd);

	return (ok ? CS_SUCCEED : CS_FAIL);
}

/*
**  LANG_EXEC - Execute one of a limited number of client language requests.
**	cp	- Application's context structure
**
**  Parameters:
**	srvproc - Open Server thread control handle to the current client
**		  connection.
**	cmd	- The language command to execute.
**	len	- The length of the command string.
**
**  Returns:
**	SRV_CONTINUE
*/
CS_RETCODE	lang_exec(cp, srvproc, cmd, len)
CS_CONTEXT	*cp;
SRV_PROC	*srvproc;
CS_CHAR		*cmd;
CS_INT		len;
{
	CS_INT		cur;
	CS_SERVERMSG	msg;

	/* Initialization.						*/
	cur = 0;			/* No parsing yet		*/

	/* Remove leading spaces.					*/
	WHITESPACE(cmd, cur, len);

	/* Was the sp_serverinfo command received?		*/
	if (memcmp(&cmd[cur], CMD_SERVERINFO, strlen(CMD_SERVERINFO)) == 0)
	{
		/* Increment the position pointer to the appropriate place. */
		cur += strlen(CMD_SERVERINFO);
		WHITESPACE(cmd, cur, len);

		/*
		** Upon successful execution, the registered procedure sends a
		** final message.   If an error occurs, however, we must
		** send the done to the client.
		*/
		return(exec_serverinfo(srvproc, cmd, cur, len));
	}
	/* See if the 'set language' command was received.		*/
	else if (memcmp(&cmd[cur], CMD_LANGUAGE, strlen(CMD_LANGUAGE)) == 0)
	{
		/* Increment the position pointer to the appropriate place. */
		cur += strlen(CMD_LANGUAGE);
		WHITESPACE(cmd, cur, len);

		/*
		** Execute the 'set language' command.
		*/
		return(exec_locale(cp, srvproc, CS_SYB_LANG, cmd, cur, len));
	}
	/* See if the 'set charset' command was received.		*/
	else if (memcmp(&cmd[cur], CMD_CHARSET, strlen(CMD_CHARSET)) == 0)
	{
		/* Increment the position pointer to the appropriate place. */
		cur += strlen(CMD_LANGUAGE);
		WHITESPACE(cmd, cur, len);

		/*
		** Execute the 'set charset' command.
		*/
		return(exec_locale(cp, srvproc, CS_SYB_CHARSET, cmd, cur, len));
	}

	/*
	** Unrecognized language command.
	**
	** Initialize the message to send to the client indicating
	** the command was not recognized.
	*/
	srv_bzero(&msg, CS_SIZEOF(CS_SERVERMSG));
	msg.msgnumber = CMD_FAIL;

	/* Fill in the message text field. 			*/
	(CS_VOID)sprintf(msg.text, "lang_exec: Unrecognized command.\n");
	msg.textlen = strlen(msg.text);

	/* Fill in the server name field.			*/
	(CS_VOID)srv_props(cp, CS_GET, SRV_S_SERVERNAME, 
		(CS_VOID *)msg.svrname, CS_SIZEOF(msg.svrname),
		&msg.svrnlen);

	/* Notify the client of the command failure. 		*/
	msg.status = (CS_FIRST_CHUNK|CS_LAST_CHUNK);
	if (srv_sendinfo(srvproc, &msg, (CS_INT)0) != CS_SUCCEED)
		(CS_VOID)fprintf(stderr, "srv_sendinfo() failed.\n");
	
	return (CS_FAIL);
}

/*
**  EXEC_SERVERINFO - Finish parsing and then execute the sp_serverinfo
**			language command that the client sent.
**
**  Parameters:
**	srvproc - A pointer to an internal thread control structure.
**		  connection.
**	cmd	- The language command to execute.
**	cur	- The current position in the language command to parse.
**	len	- The length of the command string.
**
**  Returns:
**	CS_SUCCEED | CS_FAIL
*/
CS_RETCODE	exec_serverinfo(srvproc, cmd, cur, len)
SRV_PROC	*srvproc;
CS_CHAR		*cmd;
CS_INT		cur;
CS_INT		len;
{
	CS_INT		top;
	CS_INT		retval;		/* sproc return value. 		*/

	/* 
	** Prepare to execute the sp_serverinfo registered 
	** stored procedure.
	*/
	if (srv_reginit(srvproc, CMD_SERVERINFO, CS_NULLTERM, 
		SRV_M_PNOTIFYALL) != SUCCEED) 
	{
		/* Error already raised.				*/
		return (CS_FAIL);
	}

	/* Get the first parameter. 					*/
	top = cur;
	UPTOSPACE(cmd, cur, len);

	/* Prepare the parameter for the stored procedure. 		*/
	if (srv_regparam(srvproc, CMD_PARAM1, strlen(CMD_PARAM1),
		(CS_INT)CS_CHAR_TYPE, (CS_INT)(cur - top), 
		(CS_BYTE *)&cmd[top]) != SUCCEED)
	{
		/* Error already raised.				*/
		return (CS_FAIL);
	}

	/* 
	** Check for a second parameter.
	*/
	WHITESPACE(cmd, cur, len);
	if (cmd[cur] == ',')
	{
		/* Increment the position holder.		*/
		cur++;
		WHITESPACE(cmd, cur, len);

		/* Get the second parameter. 			*/
		top = cur;
		UPTOSPACE(cmd, cur, len);

		/* Prepare the parameter for the stored procedure. 	*/
		if (srv_regparam(srvproc, CMD_PARAM2, 
			strlen(CMD_PARAM2), (CS_INT)CS_CHAR_TYPE,
			(CS_INT)(cur - top), (CS_BYTE *)&cmd[top]) 
			!= CS_SUCCEED)
		{
			/* Error already raised.			*/
			return (CS_FAIL);
		}
	}

	/* Execute the sp_serverinfo registered procedure. 	*/
	if (srv_regexec(srvproc, &retval) != CS_SUCCEED)
		return (CS_FAIL);

	return (CS_SUCCEED);
} 

/*
**  EXEC_LOCALE - Finish parsing and then execute the 'set language' or
**	'set charset' language command that the client sent.
**
**  Parameters:
**	cp	- Application's context structure.
**	srvproc - A pointer to an internal thread control structure.
**	map	- The locale map that matches language/character set name to local.
**	cmd	- The language/character set command to execute.
**	cur	- The current position in the command being parsed.
**	len	- The length of the command string.
**
**  Returns:
**	CS_SUCCEED | CS_FAIL
*/
CS_RETCODE	exec_locale(cp, srvproc, type, cmd, cur, len)
CS_CONTEXT	*cp;
SRV_PROC	*srvproc;
CS_INT		type;
CS_CHAR		*cmd;
CS_INT		cur;
CS_INT		len;
{
	CS_INT	top;

	/* Get the language/character set name.				*/
	top = cur;
	UPTOSPACE(cmd, cur, len);

	cmd[cur] = '\0';

	/* Set the thread to the new locale.				*/
	if (set_locale(cp, srvproc, type, &cmd[top], cur - top) != CS_SUCCEED)
		return (CS_FAIL);

	/* Send the final done to the client.				*/
	if (srv_senddone(srvproc, SRV_DONE_FINAL, CS_TRAN_COMPLETED, 0) 
		!= CS_SUCCEED)
	{
		(CS_VOID)fprintf(stderr, "srv_senddone() failed.\n");
	}

	return (CS_SUCCEED);
} 

/*
**  SET_LOCALE - Set a thread's locale information.
**
**  Parameters:
**	cp	- Application's context structure.
**	srvproc - A pointer to an internal thread control structure.
**	map	- The locale map that matches language/character set to a locale.
**	name	- The language or character set name to lookup.
**	len	- The length of the name.
**
**  Returns:
**	CS_SUCCEED on successful locale set, else CS_FAIL.
*/
CS_RETCODE	set_locale(cp, srvproc, type, name, len)
CS_CONTEXT	*cp;
SRV_PROC	*srvproc;
CS_INT		type;
CS_CHAR		*name;
CS_INT		len;
{
	CS_SERVERMSG	msg;
	CS_LOCALE	*lp;

	/* Initialization.						*/
	lp = (CS_LOCALE *)NULL;

	/* Allocate a locale structure for use.				*/
	if (cs_loc_alloc(cp, &lp) != CS_SUCCEED)
		return (CS_FAIL);

	/* 
	** Set the locale of the thread with the name/type provided
	*/
	if (cs_locale(cp, CS_SET, lp, type, name, len, (CS_INT *)NULL) 
		!= CS_SUCCEED)
	{
		/* Drop the locale structure allocated above.		*/
		(CS_VOID)cs_loc_drop(cp, lp);

		(CS_VOID)fprintf(stderr, 
			"%s Unable to set application to %s: %s\n", 
			"set_locale()", 
			(type == CS_SYB_LANG) ? "Language" : "Charset",
			name);
		return (CS_FAIL);
	}


	/* Set the new locale for the thread.				*/
	if (srv_thread_props(srvproc, CS_SET, SRV_T_LOCALE, lp, 
		CS_SIZEOF(lp), (CS_INT *)NULL) != CS_SUCCEED)
	{
		/* Drop the locale structure allocated above.		*/
		(CS_VOID)cs_loc_drop(cp, lp);
		return (CS_FAIL);
	}

	/* Drop the locale that was allocated.				*/
	if (cs_loc_drop(cp, lp) != CS_SUCCEED)
		return (CS_FAIL);

	/* 
	** Initialize the message to send to the client indicating
	** the locale change.
	*/
	srv_bzero(&msg, CS_SIZEOF(CS_SERVERMSG));
	msg.msgnumber = LOCALE_INFO;

	/* Fill in the message text field. 				*/
	(CS_VOID)sprintf(msg.text, "set_locale: %s changed to '%s'\n",
		(type == CS_SYB_LANG) ? "Language" : "Charset", name);
	msg.textlen = strlen(msg.text);

	/* Fill in the server name field.				*/
	(CS_VOID)srv_props(cp, CS_GET, SRV_S_SERVERNAME, 
		(CS_VOID *)msg.svrname, CS_SIZEOF(msg.svrname),
		&msg.svrnlen);

	/* Notify the client of the new locale information.		*/
	msg.status = (CS_FIRST_CHUNK|CS_LAST_CHUNK);
	if (srv_sendinfo(srvproc, &msg, (CS_INT)0) != CS_SUCCEED)
		return (CS_FAIL);

	return (CS_SUCCEED);
}
/*
**  GET_LOCALE_MAPPING - Get a thread's language/charset name from the locale.
**
**  Parameters:
**	cp	- Application's context structure.
**	srvproc - A pointer to an internal thread control structure.
**	map	- The locale map that matches lang/charset to a locale.
**	name	- The buffer in which to put the option name.
**	len	- The length of the buffer.
**
**  Returns:
**	CS_SUCCEED on successful locale set, else CS_FAIL
*/
CS_RETCODE	get_locale_mapping(cp, srvproc, type, name, len)
CS_CONTEXT	*cp;
SRV_PROC	*srvproc;
CS_INT		type;
CS_CHAR		*name;
CS_INT		len;
{
	CS_INT		outlen;
	CS_LOCALE	*lp;

	/* Initialization.						*/
	lp = (CS_LOCALE *)NULL;

	/* Allocate a locale structure for use.				*/
	if (cs_loc_alloc(cp, &lp) != CS_SUCCEED)
		return (CS_FAIL);

	/* Get the locale for the client's thread.			*/
	if (srv_thread_props(srvproc, CS_GET, SRV_T_LOCALE, lp, 
		CS_SIZEOF(lp), (CS_INT *)NULL) != CS_SUCCEED)
	{
		/* Drop the locale structure allocated above.		*/
		(CS_VOID)cs_loc_drop(cp, lp);
		return (CS_FAIL);
	}

	/* Get the locale name from the locale structure. 		*/
	if (cs_locale(cp, CS_GET, lp, type, name, len, &outlen)
		!= CS_SUCCEED)
	{ 
		/* Drop the locale structure allocated above.		*/
		(CS_VOID)cs_loc_drop(cp, lp);
		return (CS_FAIL);
	}

	/* Release the context structure already allocated.		*/
	if (cs_loc_drop(cp, lp) != CS_SUCCEED)
		return (CS_FAIL);

	return (CS_SUCCEED);
}
