
/*
**	Sybase Open Server Version 10.0
**	Confidential Property of Sybase, Inc.
**	(c) Copyright Sybase, Inc. 1993
**	All rights reserved
**
** lang.c
**
** This sample program demonstrates a simple
** stand-alone Open Server application.  It installs a
** SRV_LANGUAGE event handler and responds to client commands with
** an informational message that contains the language command 
** received. The program also installs a SRV_CONNECT event handler,
** which sends to the client an informational message containing the
** user name and password being used to connect.
** This sample further illustrates the use of Open Server and 
** CS-Library error handler routines to report errors.
**
** Syntax:
**
** 	lang [server name] [trace flags]
**
** If a server name argument is given, it will be used. Otherwise,
** the value of the DSLISTEN environment variable is used.
** The optional trace flags result in diagnostic information being
** written to the Open Server log file. The name of the logfile
** will be <server_name>.log.
*/

#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>

#include	<ospublic.h>
#include	<ossample.h>

#if	(USE_SCCSID)
static char	Sccsid[] = {"%Z% %M% %I% %G%"};
#endif	/* (USE_SCCSID) */

/*
** Local defines.
*/
#define	MY_LOGIN_MSG 	"User '%s' logged in with password '%s'."
#define	MY_LANG_MSG 	"Language handler called with string '%s'."
#define	INFO_MSG1	(CS_INT)5555
#define	INFO_MSG2	(CS_INT)6666

/*
** Prototypes.
*/
CS_RETCODE	CS_PUBLIC connect_handler PROTOTYPE ((
	SRV_PROC	*sp
	));
CS_RETCODE	CS_PUBLIC lang_handler PROTOTYPE ((
	SRV_PROC	*sp
	));
CS_VOID		done_error PROTOTYPE ((
	SRV_PROC	*sp
	));

int		main( argc, argv )
CS_INT		argc;
CS_CHAR		**argv;
{
	CS_CONTEXT	*cp;		/* Context structure. */
	SRV_SERVER	*server;	/* the Server process itself. */
	CS_CHAR		logname[CS_MAX_CHAR];	/* The log file name. */
	CS_CHAR		sname[CS_MAX_CHAR];	/* The server name. */


	/*
	** Get a context.
	*/
	if( cs_ctx_global(CS_VERSION_100, &cp) == CS_FAIL )
	{
		(CS_VOID)fprintf(stderr,
		"ERROR: cs_ctx_global() failed!\n");

		cs_ctx_drop(cp);

		exit(1);
	}

	/*
	** Install a CS-Library error handler.
	*/
	if( cs_config(cp, CS_SET, CS_MESSAGE_CB, (CS_VOID *)cs_err_handler,
		CS_UNUSED, (CS_INT *)NULL) == CS_FAIL )
	{
		(CS_VOID)fprintf(stderr,
			"ERROR: cs_config(CS_MESSAGE_CB) failed!\n");

		cs_ctx_drop(cp);

		exit(1);
	}

	/*
	** Initialize Open Server.
	*/
	if( srv_version(cp, CS_VERSION_100) == CS_FAIL )
	{
		(CS_VOID)fprintf(stderr, "ERROR: srv_version() failed!\n");

		cs_ctx_drop(cp);

		exit(1);
	}

	/*
	** Process the command-line arguments.
	*/
	if( proc_args(cp, argc, argv, sname, CS_MAX_CHAR, (CS_CHAR *)NULL,
		(CS_INT)0) == CS_FAIL )
	{
		/*
		** An error was already raised.
		*/
		cs_ctx_drop(cp);

		exit(1);
	}

	/* 
	** Install the Open Server error handler.
	*/
	if( srv_props(cp, CS_SET, SRV_S_ERRHANDLE,
		(CS_VOID *)server_err_handler, sizeof(CS_VOID *),
		(CS_INT *)NULL) == CS_FAIL )
	{
		/*
		** An error was already raised.
		*/
		cs_ctx_drop(cp);

		exit(1);
	}

	
	/* 
	** Define the log file.
	*/
	if( sname[0] != (CS_CHAR)'\0' )
	{
		(CS_VOID)strcpy(logname, sname);
		(CS_VOID)strcat(logname, ".log");
	}
	else
	{
		(CS_VOID)strcpy(logname, "os.log");
	}

	if( srv_props(cp, CS_SET, SRV_S_LOGFILE, logname,
		CS_NULLTERM, NULL) == CS_FAIL )
	{
		/*
		** An error was already raised.
		*/
		cs_ctx_drop(cp);

		exit(1);
	}

	/* 
	** Initialize Open Server.
	*/
	if( (server = srv_init((SRV_CONFIG *)NULL, sname, 
			CS_NULLTERM)) == (SRV_SERVER *)NULL )
	{
		/*
		** An error was already raised.
		*/
		cs_ctx_drop(cp);

		exit(1);
	}

	/*
	** Install the event handlers.
	*/
	srv_handle(server, SRV_START, stop_regproc);
	srv_handle(server, SRV_CONNECT, connect_handler);
	srv_handle(server, SRV_LANGUAGE, lang_handler);

	/*
	** Print a version message.
	*/
	print_version(cp, sname);

	/*
	** Start up Open Server.
	*/
	
	if( srv_run((SRV_SERVER *)NULL) == CS_FAIL )
	{
		/*
		** An error was already raised.
		*/
		cs_ctx_drop(cp);

		exit(1);
	}

	/*
	** Clean up and exit.
	*/
	cs_ctx_drop(cp);

	exit(0);
}

/*
** connect_handler
**
** This routine is the SRV_CONNECT event handler this application uses.
** Here we get the connecting client thread's user name and password,
** and echo them back to the client via an informational message.
*/
CS_RETCODE	CS_PUBLIC connect_handler(sp)
SRV_PROC	*sp;
{
	CS_CONTEXT	*cp;			/* Context structure. */
	CS_SERVERMSG	msg;			/* The message we'll send. */
	CS_CHAR		user[CS_MAX_NAME];	/* The client's user name. */
	CS_CHAR		pwd[CS_MAX_NAME];	/* The client's password. */
	CS_CHAR		sname[CS_MAX_NAME];	/* The server name. */
	CS_INT		slen;			/* The server name length. */
	CS_INT		ulen;			/* The user name length. */
	CS_INT		plen;			/* The password length. */
	CS_BOOL		ok;

	/*
	** Initialization.
	*/
	ok = CS_TRUE;
	srv_bzero(&msg, sizeof(msg));

	/*
	** Get the CS_CONTEXT we're using.
	*/
	if( cs_ctx_global(CS_VERSION_100, &cp) == CS_FAIL )
	{
		/*
		** An error was already raised.
		*/
		done_error(sp);

		return CS_FAIL;
	}

	/*
	** Get the name of the server.
	*/
	if( srv_props(cp, CS_GET, SRV_S_SERVERNAME, sname, 
			CS_MAX_NAME, &slen) == CS_FAIL )
	{
		/*
		** An error was already raised.
		*/
		done_error(sp);

		return CS_FAIL;
	}

	/*
	** Get the client thread's user name.
	*/
	if( srv_thread_props(sp, CS_GET, SRV_T_USER, user,
			CS_MAX_NAME, &ulen) == CS_FAIL )
	{
		/*
		** An error was already raised.
		*/
		done_error(sp);

		return CS_FAIL;
	}

	/*
	** Get the client thread's password.
	*/
	if( srv_thread_props(sp, CS_GET, SRV_T_PWD, pwd,
			CS_MAX_NAME, &plen) == CS_FAIL )
	{
		/*
		** An error was already raised.
		*/
		done_error(sp);

		return CS_FAIL;
	}

	/*
	** Null terminate the user and password strings.
	*/
	user[ulen] = (CS_CHAR)'\0';
	pwd[plen] = (CS_CHAR)'\0';

	/*
	** Initialize the message we're sending. We'll
	** pick an arbitrary message number, and copy the message
	** string into the CS_SERVERMSG structure.
	*/
	msg.msgnumber = INFO_MSG1;

	/*
	** Fill in the message text field.
	*/
	(CS_VOID)sprintf(msg.text, MY_LOGIN_MSG, user, pwd);
	msg.textlen = strlen(msg.text);

	/*
	** Fill in the server name field as well.
	*/
	(CS_VOID)strncpy(msg.svrname, sname, slen);
	msg.svrnlen = slen;

	/*
	** Send the message to the client.
	*/
	msg.status = (CS_FIRST_CHUNK|CS_LAST_CHUNK);
	if( (ok) && (srv_sendinfo(sp, &msg, CS_TRAN_UNDEFINED) == CS_FAIL) )
	{
		/*
		** An error was already raised.
		*/
		done_error(sp);

		return CS_FAIL;
	}

	/*
	** Send a done to complete the command.
	*/
	if( srv_senddone(sp,  SRV_DONE_FINAL,
			CS_TRAN_COMPLETED, (CS_INT)0) == CS_FAIL )
	{
		/*
		** An error was already raised.
		*/
		return CS_FAIL;
	}

	/*
	** All done.
	*/
	return CS_SUCCEED;
}


/*
** lang_handler
** This routine is the SRV_LANGUAGE event handler. All we do here
** is get the incoming language string, and send it back to the
** client via an informational message.
*/
CS_RETCODE	CS_PUBLIC lang_handler(sp)
SRV_PROC	*sp;
{
	CS_CONTEXT	*cp;			/* Context structure. */
	CS_SERVERMSG	msg;			/* The message we'll send. */
	CS_CHAR		sname[CS_MAX_NAME];	/* The server name. */
	CS_INT		slen;			/* The server name length. */
	CS_CHAR		*cmd;
	CS_INT		len;			/* the length of the message. */

	/*
	** Initialization.
	*/
	srv_bzero(&msg, sizeof(msg));


	/*
	** Get the CS_CONTEXT structure.
	*/
	if( cs_ctx_global(CS_VERSION_100, &cp) == CS_FAIL )
	{
		/*
		** An error was already raised.
		*/
		done_error(sp);

		return CS_FAIL;
	}

	/*
	** Get the name of the server.
	*/
	if( srv_props(cp, CS_GET, SRV_S_SERVERNAME, sname, 
			CS_MAX_NAME, &slen) == CS_FAIL )
	{
		/*
		** An error was already raised.
		*/
		done_error(sp);

		return CS_FAIL;
	}


	/*
	** Get the length of the language string.
	*/
	if( (len = srv_langlen(sp)) == -1 )
	{
		/*
		** An error was already raised.
		*/

		done_error(sp);

		return CS_FAIL;
	}

	/*
	** Allocate enough space to hold the language string.
	*/
	if( (cmd = (CS_CHAR *)srv_alloc(len + 1)) == (CS_CHAR *)NULL )
	{
		/*
		** An error was already raised.
		*/
		done_error(sp);

		return CS_FAIL;
	}


	/*
	** Get the language string itself.
	*/
	if( srv_langcpy(sp, 0, -1, cmd) == -1 )
	{
		/*
		** An error was already raised.
		*/
		done_error(sp);

		return CS_FAIL;
	}

	/*
	** We may want to truncate this string so that it
	** fits in the message text buffer.
	*/
	if( len > (CS_INT)(CS_MAX_MSG - strlen(MY_LANG_MSG) ))
	{
		cmd[CS_MAX_MSG - strlen(MY_LANG_MSG)] = (CS_CHAR)'\0';
	}

	/*
	** Place the message string in place.
	*/
	(CS_VOID)sprintf(msg.text, MY_LANG_MSG, cmd);
	msg.textlen = strlen(msg.text);

	/*
	** Let's make up a message number.
	*/
	msg.msgnumber = INFO_MSG2;

	/*
	** Fill in the server name field as well.
	*/
	(CS_VOID)strncpy(msg.svrname, sname, slen);
	msg.svrnlen = slen;

	/*
	** Send the message to the client.
	*/
	msg.status = (CS_FIRST_CHUNK|CS_LAST_CHUNK);
	if( srv_sendinfo(sp, &msg, CS_TRAN_UNDEFINED) == CS_FAIL )
	{
		/*
		** An error was already raised.
		*/
		done_error(sp);

		return CS_FAIL;
	}

	/*
	** Let's clean up.
	*/
	srv_free(cmd);

	/*
	** And finally, send a done to complete the command.
	*/
	srv_senddone(sp, SRV_DONE_FINAL, CS_TRAN_COMPLETED, (CS_INT)0);

	/*
	** All done.
	*/
	return CS_SUCCEED;
}

/*
** done_error
**
** This routine is used to send a done-error message to client threads
** when some client-originated login or command has failed.
**
*/

CS_VOID		done_error(sp)
SRV_PROC	*sp;
{

	/*
	** All we need to do is send the done. If this fails,
	** print an error to the screen and return.
	*/
	if( srv_senddone(sp, SRV_DONE_ERROR | SRV_DONE_FINAL,
		CS_TRAN_COMPLETED, (CS_INT)0) == CS_FAIL )
	{
		(CS_VOID)fprintf(stderr, "lang: Failed to send a done!\n");
	}

	return;
}


